import { format } from 'd3';
import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import ReactDropdown from 'react-dropdown';
import { config } from './config';
import { useData } from './useData';
import { getTop10BarData } from './sumData';
import { getSumData } from './sumDataByYear';
import { getLabel, ValueAttributes, FieldAttributes } from './AttributesDefs';
import { getUsAtlas } from './getUsAtlas';
import { Dropdown } from './Dropdown';
import { BarColorMark } from './BarColorMark/index.js';
import { DotColorMark } from './DotColorMark/index.js';
import { MapColorMark } from './MapColorMark/index.js';
import { LineColorMark } from './LineColorMark/index.js';

//Part I: Global values
const width = 960;
const menuHeight = 60;
const height = 500 - menuHeight;
const margin = { top: 10, right: 30, bottom: 65, left: 90 };
const xAxisLabelOffset = 30;
const yAxisLabelOffset = 50;
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;
const halfWidth = (innerWidth - 20) / 2;
const halfHeight = (innerHeight - 20) / 2;
const halfMarginX = 30;
const halfMarginY = 40;
//Interaction related
const fadeOpacity = 0.2;

//Part II: Main functiond
const App = () => {
  const data = useData();
  const usAtlas = getUsAtlas();
  const [hoveredValue, setHoveredValue] = useState(null);

  //Step 0: Menu with attribute selections
  //xValue and yValues are defined here
  const initialXAttribute = 'Week';
  const [xAttribute, setXAttribute] = useState(initialXAttribute);
  const xValue = d => d[xAttribute];
  //  xLabel is fixed for X
  const xAxisLabel = getLabel(xAttribute, FieldAttributes);
  
  const initialYAttribute = 'ILI_Percent';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  const yValue = d => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute, ValueAttributes);

  if (!data || !usAtlas) {
    return <pre>Loading...</pre>;
  }
  //console.log(yAxisLabel);
  //console.log(xAxisLabel);
  //console.log(xAttribute);

  const siFormat = format('.2s');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');

  //Step 1: Derived data from here for plot:4-1 Line chart
  //Move into LineColorMark
  const colorValue = d => d.Year;
  const sizeValue = d => d.ILI_Total;
  const tooltipValue = d => d.ILI_Total;
  const sumDataByYear = getSumData(data, yValue);
  console.log('Called sumData by year.');

  //Step 2: US map for all states plot: 4-2
  //Move into MapColorMark\
  //console.log(usAtlas);
  //obviously svg is not available here!
//  const svg = select('svg');

  //Step 3: Derived data from here for plot: 4-3 Bar chart
  const top10BarData = getTop10BarData(data, yValue);
//  console.log(top10BarData);
  //Move into BarColorMark\
  console.log('Called getTop10Bar by year.');

  const yBarValue = d => d.State;
  const xBarValue = d => d.Tvalue;

  //Step 4: Derived line plot plot: 4-4 Curve chart
  //  const xAxisTickFormat = timeFormat('%a');
  //Aggregate data by weeks

  //Step 5: Draw plots using svg, other controls: ReactDropdown
  return (
    <>
      <div className="menus-container">
        <span className="dropdown-label">Choose a data field:</span>
        <ReactDropdown
          options={ValueAttributes}
          value={yAttribute}
          onChange={({ value }) => setYAttribute(value)}
        />
      </div>
      <svg width={width} height={height}>
        <g
          transform={`translate(${margin.left + halfWidth + halfMarginX},${
            margin.top
          }) scale(0.995 0.995)`}
        >
          <svg
            width={halfWidth}
            height={halfHeight + 20}
            viewBox={`0 0 ${width} ${height + 60}`}
            zoomAndPan={'zoomAndPan'}
          >
            <MapColorMark atlas={usAtlas} />
          </svg>
        </g>
        <g
          transform={`translate(${margin.left},${margin.top +
            halfHeight +
            halfMarginY})`}
        >
          <BarColorMark
            data={top10BarData}
            xValue={xBarValue}
            yValue={yBarValue}
            innerWidth={halfWidth}
            innerHeight={halfHeight}
            xAxisLabel={yAxisLabel}
            xAxisTickFormat={xAxisTickFormat}
          />
        </g>
        <g transform={`translate(${margin.left},${margin.top})`}>
          <LineColorMark
            data={sumDataByYear}
            xValue={xValue}
            yValue={yValue}
            sizeValue={sizeValue}
            colorValue={colorValue}
            innerWidth={halfWidth}
            innerHeight={halfHeight}
            xAxisLabel={xAxisLabel}
            yAxisLabel={yAxisLabel}
            xAxisTickFormat={xAxisTickFormat}
            tooltipValue={tooltipValue}
            onHover={setHoveredValue}
            hoveredValue={hoveredValue}
            fadeOpacity={fadeOpacity}
          />
        </g>
        <g
          transform={`translate(${margin.left +
            halfWidth +
            halfMarginX},${margin.top + halfHeight + halfMarginY})`}
        >
          <DotColorMark
            data={data}
            xValue={xValue}
            yValue={yValue}
            sizeValue={sizeValue}
            colorValue={colorValue}
            innerWidth={halfWidth}
            innerHeight={halfHeight}
            xAxisLabel={xAxisLabel}
            yAxisLabel={yAxisLabel}
            xAxisTickFormat={xAxisTickFormat}
            tooltipValue={tooltipValue}
          />
        </g>
      </svg>
    </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
